//
//  NSFileManager_SenAdditions.m
//  CVL
//
//  Created by William Swats on Thu Oct 23 2003.
//  Copyright (c) 2003 Sente SA. All rights reserved.
//

#import "NSFileManager_SenAdditions.h"

#import <SenFoundation/SenFoundation.h>


@implementation NSFileManager (SenAdditions)


- (BOOL) senFileExistsAtPath:(NSString *)aPath isDirectory:(BOOL *)isADirectory
    /*" This checks to see if the file path in the argument aPath points to an 
        existing file or directory. If it does then this method returns YES; 
        otherwise NO is returned (i.e. aPath is nil, empty or does not exists). 
        This method follows links.
    
        We are using this method in place of #{-fileExistsAtPath:isDirectory:} in 
        NSFileManager since that method does not follow links.
    "*/
{
    NSFileManager   *fileManager = nil;
    NSDictionary    *fileAttributes = nil;
    NSString        *aFileType = nil;
    BOOL            theFileExists = NO;
    
    SEN_ASSERT_CONDITION((isADirectory != NULL));
    
    if ( isNotEmpty(aPath) ) {
        fileManager = [NSFileManager defaultManager];
        fileAttributes = [fileManager fileAttributesAtPath:aPath 
                                                    traverseLink:YES];
        if ( isNotEmpty(fileAttributes) ) {
            theFileExists = YES;
            aFileType = [fileAttributes objectForKey:NSFileType];
            if ( (aFileType != nil) && (aFileType == NSFileTypeDirectory) ) {
                *isADirectory = YES;
            } else {
                *isADirectory = NO;
            }
        }        
    }
    return theFileExists;
}

- (BOOL) senDirectoryExistsAtPath:(NSString *)aPath
    /*" This checks to see if the file path in the argument aPath points to an 
        existing directory. If it does then this method returns YES; otherwise NO is
        returned (i.e. aPath is nil, empty, does not exists or is not a directory). 
        This method follows links.
            
        See also #{-senFileExistsAtPath:isDirectory:} for more information.
    "*/
{
    BOOL            theFileExists = NO;
    BOOL            isAdirectory = NO;

    if ( isNotEmpty(aPath) ) {
        theFileExists = [self senFileExistsAtPath:aPath
                                      isDirectory:&isAdirectory];
        if ( theFileExists && isAdirectory ) {
            return YES;
        }
    }
    return NO;
}


- (BOOL) senFileExistsAtPath:(NSString *)aPath
    /*" This checks to see if the file path in the argument aPath points to an 
        existing file or directory. If it does then this method returns YES; 
        otherwise NO is returned (i.e. aPath is nil, empty or does not exists).
        This method follows links.
    
        We are using this method in place of #{-fileExistsAtPath:} in 
        NSFileManager since that method does not follow links even though the
        documentation says it does. This was the case with Xcode on Jaguar says
        William Swats as of 22-Oct-2003.
    "*/
{
    NSFileManager   *fileManager = nil;
    NSDictionary    *fileAttributes = nil;
    
    if ( isNotEmpty(aPath) ) {
        fileManager = [NSFileManager defaultManager];
        fileAttributes = [fileManager fileAttributesAtPath:aPath 
                                                    traverseLink:YES];
        if ( isNotEmpty(fileAttributes) ) {
            return YES;
        }        
    }
    return NO;
}


@end
