/*$Id: SenPathUtilities.m,v 1.4 2003/07/25 16:07:16 phink Exp $*/

// Copyright (c) 1997-2003, Sen:te (Sente SA).  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenPathUtilities.h"

#if !defined(GNUSTEP) && !defined(RHAPSODY)
NSString *FolderManagerException = @"FolderManagerException";
NSString *CFURLException = @"CFURLException";


// see Folders.h for parameter constants
// /System/Library/Frameworks/CoreServices.framework/Versions/A/Frameworks/CarbonCore.framework/Versions/A/Headers/Folders.h

// value for vRefNum
// enum {
//     kOnSystemDisk                 = -32768L, /* previously was 0x8000 but that is an unsigned value whereas vRefNum is signed*/
//     kOnAppropriateDisk            = -32767, /* Generally, the same as kOnSystemDisk, but it's clearer that this isn't always the 'boot' disk.*/
     /* Folder Domains - Carbon only.  The constants above can continue to be used, but the folder/volume returned will*/
     /* be from one of the domains below.*/
//     kSystemDomain                 = -32766, /* Read-only system hierarchy.*/
//     kLocalDomain                  = -32765, /* All users of a single machine have access to these resources.*/
//     kNetworkDomain                = -32764, /* All users configured to use a common network server has access to these resources.*/
//     kUserDomain                   = -32763, /* Read/write. Resources that are private to the user.*/
//     kClassicDomain                = -32762 /* Domain referring to the currently configured Classic System Folder*/
// };

NSString *SenSearchPathForDirectoriesInDomains(OSType folderType, short vRefNum)
{
    OSErr err;
    FSRef folderRef;
    //UInt8 localPath[4096] = "";
    CFURLRef anURL;
    CFStringRef aPath;

    err = FSFindFolder (vRefNum, folderType, kDontCreateFolder, &folderRef);
    if (err != 0) {
        [NSException raise: FolderManagerException format: @"Folder Manager error (%d)", err];
    }
    anURL = CFURLCreateFromFSRef (kCFAllocatorSystemDefault, &folderRef);
    //err = FSRefMakePath (&folderRef, localPath, 4096);
    if (anURL == NULL) {
        [NSException raise: CFURLException format: @"CFURLCreateFromFSRef"];
    }
    aPath = CFURLCopyFileSystemPath (anURL, kCFURLPOSIXPathStyle);
    CFRelease (anURL);
    return [(NSString *)aPath autorelease];
}

#endif

@implementation NSString (SenPathUtilities)
+ (NSString *) temporaryPathWithName:(NSString *) aName forApplicationIdentifier:(NSString *) anApplicationIdentifier
{
	NSArray *temporaryPath = [NSArray arrayWithObjects:
		NSTemporaryDirectory(),
		anApplicationIdentifier,
		aName,
		nil];

	return [NSString pathWithComponents:temporaryPath];
}


+ (NSString*) uniqueTemporaryPathWithName:(NSString *) aName forApplicationIdentifier:(NSString *) anApplicationIdentifier
{
	return [[self temporaryPathWithName:aName forApplicationIdentifier:anApplicationIdentifier] stringByAppendingPathComponent:[[NSProcessInfo processInfo] globallyUniqueString]];
}
@end

